# Generated by Django 4.2.23 on 2026-01-24 15:50

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aatps", "0002_campaignkillmail_ship_type_id_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="campaignkillmail",
            name="ship_group_name",
            field=models.CharField(default="", max_length=255),
        ),
    ]
